<?php

namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\ChecklistAgendService;
use FM\Traits\TResponse;

class ChecklistAgendController extends FMCrud {
    use TResponse;

    protected $ChecklistAgendService;

    public function __construct($crudService, $environment, ChecklistAgendService $ChecklistAgendService) {
        parent::__construct($crudService, $environment);
        $this->ChecklistAgendService = $ChecklistAgendService;
    }

    public function listTask(DTO\Request $request, DTO\Response $response){
        $params = $request->getParameters();
        $var = $this->ChecklistAgendService->listTask($params);
        $dtsVal = new DataSet("task", $var);
        $response->addDataSet($dtsVal);
    }

    public function infoTask(DTO\Request $request, DTO\Response $response){
        try {
            $data = $this->ChecklistAgendService->infoTask($request->getParameters());
            $this->sendResponse($data, 200);
          } catch (Exception $ex) {
              $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function taskOfTheEmployee(DTO\Request $request, DTO\Response $response){
        try {
            $data = $this->ChecklistAgendService->taskOfTheEmployee($request->getParameters());
            $this->sendResponse($data, 200);
          } catch (Exception $ex) {
              $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function transferTask(DTO\Request $request, DTO\Response $response){
        try {
            $data = $this->ChecklistAgendService->transferTask($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getLocals(DTO\Request $request, DTO\Response $response) {

        $id = $request->getParameters();

        $locals = $this->ChecklistAgendService->getLocals($id);
        $dtsVal = new DataSet('locals', $locals);
        $response->addDataSet($dtsVal);
    }
    
    public function listChecklistAgend(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $checklistAgend = $this->ChecklistAgendService->listChecklistAgend($params);
        $dtsVal = new DataSet('ChecklistAgends', $checklistAgend);
        $response->addDataSet($dtsVal);
    }
    
    public function innerJoinChecklist(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $checklistAgend = $this->ChecklistAgendService->innerJoinChecklist($params);
        $dtsVal = new DataSet('ChecklistAgends', $checklistAgend);
        $response->addDataSet($dtsVal);
    }
    
    public function updateChecklistAgend(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        $changedAttributes = $this->ChecklistAgendService->updateChecklistAgend($params);
        $dtsVal = new DataSet('UpdatedValues', $changedAttributes);
        $response->addDataSet($dtsVal);
    }
}

?>