<?php
namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class Fm00005000VistoriasFactory {

    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

   // Busca e filtra vistorias
    public function getVistorias($hierarchyParam) {
        $sql = "SELECT DISTINCT GCHK.NRGERENCIAMENTOCHK, CA.NRCHECKLISTAGEND, GCHK.NMGERENCIAMENTOCHK,CLIENTE.CDCLIENTE, CLIENTE.NMRAZSOCCLIE, CA.DSJUSTCANCELAMENTO, TA.DSOBSTAREFAAGEND, GCHK.IDOBRIGAOBSERVACAO, GCHK.NRLOCALCLIENTE, GCHK.CDOPERADOR,
        GCHK.DTHORAAGEND, FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI, CA.IDSITUACAOAGEND,
            DECODE(CA.IDSITUACAOAGEND, '4',  CASE GCHK.IDSITUACAOGERCHK WHEN '1' THEN 'Aguardando Vistoria'
                                        WHEN '2' THEN 'Em andamento'
                                        WHEN '3' THEN 'Avaliado'
                                        ELSE NULL END, 'Tarefa Pendente') AS IDSITUACAOGERCHK
                FROM FCM_GERENCIAMENTOCHK GCHK
                INNER JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
                INNER JOIN OPERADOR FUNC
                ON FUNC.CDOPERADOR = GCHK.CDOPERADOR
                INNER JOIN FCM_LOCALCLI LCP
                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                INNER JOIN FCM_CHECKLISTAGEND CA
                ON CA.NRCHECKLISTAGEND = GCHK.NRCHECKLISTAGEND 
                INNER JOIN CLIENTE
                ON LC.CDCLIENTE = CLIENTE.CDCLIENTE
                INNER JOIN FCM_TAREFAAGEND TA
                ON CA.NRCHECKLISTAGEND = TA.NRCHECKLISTAGEND
                    WHERE GCHK.CDOPERADOR IN ({$hierarchyParam})
                          ORDER BY DTHORAAGEND DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    // Busca as informações do TreeGrid
    public function getCriteriosGrid ($params){
        $sql = "SELECT DISTINCT 'I'||FCM_ATIVIDADECHK.NRATIVIDADECHK AS CODIGO, NMATIVIDADE AS NOME, 
        NULL AS CODIGOPAI, 'Item' AS TIPO, NULL AS IDMARCADO,
         FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_ATIVIDADECHK.NRATIVIDADECHK AS NR
          FROM FCM_ATIVIDADECHK
          JOIN FCM_TAREFACHK
           ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
          JOIN FCM_ATIVIDADE
            ON FCM_ATIVIDADECHK.NRATIVIDADE = FCM_ATIVIDADE.NRATIVIDADE
          JOIN FCM_TAREFAGERCHK
            ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
          JOIN FCM_GERENCIAMENTOCHK 
            ON FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
           AND FCM_GERENCIAMENTOCHK.NRCHECKLIST = FCM_TAREFACHK.NRCHECKLIST
          WHERE FCM_TAREFAGERCHK.IDATIVO = 'S'
            AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
        UNION ALL
        SELECT 'A'||FCM_TAREFAGERCHK.NRTAREFAGERCHK AS CODIGO, FCM_TAREFA.NMTAREFA AS NOME, 
              'I'||FCM_ATIVIDADECHK.NRATIVIDADECHK AS CODIGOPAI, 'Atividade' AS TIPO, NULL AS IDMARCADO, 
              FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_TAREFAGERCHK.NRTAREFAGERCHK AS NR
              FROM FCM_TAREFAGERCHK
              JOIN FCM_TAREFACHK
                ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
              JOIN FCM_ATIVIDADECHK
                ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
              JOIN FCM_TAREFA
                ON FCM_TAREFACHK.NRTAREFA = FCM_TAREFA.NRTAREFA
              WHERE FCM_TAREFAGERCHK.IDATIVO = 'S'
                AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
          UNION ALL
              SELECT 'C'||FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK AS CODIGO, FCM_CRITERIOTARGERCHK.NMCRITERIOTARGERCHK AS NOME, 
                     'A'||FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK AS CODIGOPAI, 'Critério' AS TIPO, FCM_CRITERIOTARGERCHK.IDMARCADO, 
                     FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK AS NR
              FROM FCM_CRITERIOTARGERCHK
              JOIN FCM_TAREFAGERCHK
                ON FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK = FCM_TAREFAGERCHK.NRTAREFAGERCHK
              WHERE FCM_CRITERIOTARGERCHK.IDATIVO = 'S'
                AND FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

      return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    // Atualiza o campo IDMARCADO do critério para S ou N
    public function salvarCriterios($param) {
        $sql = "UPDATE FCM_CRITERIOTARGERCHK
                   SET IDMARCADO    = '$param[IDMARCADO]',
                       DTULTATU     = SYSDATE,
                       NRORGULTATU  = $param[NRORGULTATU], 
                       CDOPERULTATU = $param[CDOPERULTATU]
                 WHERE NRCRITERIOTARGERCHK = $param[NRCRITERIOTARGERCHK]";
        
        $std = $this->connection->prepare($sql);
        $std->execute();

        $sql1 = "UPDATE FCM_GERENCIAMENTOCHK
                    SET IDSITUACAOGERCHK = '2',
                        DTULTATU         = SYSDATE,
                        NRORGULTATU      = $param[NRORGULTATU],
                        CDOPERULTATU     = $param[CDOPERULTATU]
                  WHERE NRGERENCIAMENTOCHK = $param[NRGERENCIAMENTOCHK]";
        
        $std1 = $this->connection->prepare($sql1);
        $std1->execute();
    }

    // Atualiza o campo de observação da vistoria
    public function salvarObservacao($params) {
        $sql = "UPDATE FCM_TAREFAGERCHK
                   SET DSOBSTAREFAGERCHK = :DSOBSTAREFAGERCHK,
                       DTULTATU          = SYSDATE,
                       NRORGULTATU       = :NRORGULTATU, 
                       CDOPERULTATU      = :CDOPERULTATU
                 WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    // Atualiza campo IDSITUACAOGERCHK como Avaliado
    public function finalizarVistoria($parametros) {
        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = '3',
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU, 
                CDOPERULTATU = :CDOPERULTATU
                  WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

}
